from enum import Enum
from pydantic import BaseModel


class DebugOutcome(str, Enum):
    PASS = "PASS"
    FAIL = "FAIL"
    NOT_APPLICABLE = "NOT_APPLICABLE"


class DebugAnalysisResult(BaseModel):
    outcome: DebugOutcome
    explanation: str
